#!/usr/bin/perl -w 
# functions.pl - Other functions for webmin-ldap-skolelinux
#
# $Id: functions.pl,v 1.8.2.1 2004/01/17 14:04:40 andreas Exp $


use strict;
use FileHandle;
use IPC::Open2;

my @scriptdirs = qw(/etc/webmin/ldap-users /usr/share/webmin/ldap-users .);

# Call the first executable script in @scriptdirs.  This make it easy
# to change locally which script to call.
sub run_script {
    my ($scriptname, @args) = @_;

    my $done = 0;
    my $retval = 1;
    
    my $output = "<pre>\n";
    for my $scriptdir (@scriptdirs) {
	my $path = "$scriptdir/$scriptname";
	if ( -x $path) {
	    # from system(), 0 means success.  invert before returning it.
	    my $output .= `$path @args 2>&1`;
	    $retval = ! $?; 
	    $done = 1;
	    last;
	}
    }
    if ( ! $done ) {
	$output .= $text{'error'}.": Unable to find any executable ".
	      "script $scriptname in " . join(" ", @scriptdirs) . ".\n";
	$retval = undef;
    }
    $output .= "</pre>\n";
    return ($retval, $output);
}

# Create a users home directory.
sub create_dir {
    my ($homedir, $uidNumber, $gidNumber) = @_;

    return undef unless $uidNumber;

    return run_script("createhomedir", $homedir, $uidNumber, $gidNumber);
}

# Delete a users home directory.
sub delete_dir {
    my ($homedir) = @_;

    return undef unless $homedir;

    return run_script("removehomedir", $homedir);
}

# Check if username (uid) is a valid string.
sub valid_uid {
    my ($uid) = @_;

    return 0 unless $uid;
    return 0 unless (length($uid) < 9);
    return $uid =~ m/^[a-z][a-z0-9]*$/;
}

# Check if groupname (gid) is a valid string.
sub valid_gid {
    my ($gid) = @_;

    return 0 unless $gid;
    return 0 unless (length($gid) < 9);
    return $gid =~ m/^[a-z][a-z0-9]*$/;
}


sub change_samba {
    my ($student_id, $newpass, $rootpass) =@_;
    
    # try to add the user to samba
    open2(*Reader, *Writer, "/usr/bin/smbpasswd -a -s $student_id" );
    print Writer "$newpass\n$newpass\n";
    select((select(Reader), $|=1)[0]); # make the filehandle hot
    my $output = <Reader>;
    close READER;
    close WRITER;
    
    # if it failed we change the rootpw samba uses, and try again
    # we assume the ldap admin pw has changed (or it is the first time)
    my $retmsg = "";
    if($?) {
        my $ret = `/usr/bin/smbpasswd -w $rootpass`;
        if($?) {
            $retmsg .= "Failed to update ldap admin passwd in samba!";
        }
        open2(*Reader, *Writer, "/usr/bin/smbpasswd -a -s $student_id" );
        print Writer "$newpass\n$newpass\n";
        select((select(Reader), $|=1)[0]); # make the filehandle hot
        $ret = <Reader>;
        close READER;
        close WRITER;
        if($?) {
            $retmsg .= "Failed to add user $student_id to samba!\n";
        }
    }
    
    if($retmsg) {
        return $retmsg;
    } else {
        return "Added user $student_id to samba\n";
    }
}


1;
